==> django_simpleui_dev-2022.6.22-py3-none-any.whl: simpleui/admin.py <==
import json
import traceback
from functools import update_wrapper
from django.contrib import admin
from django.db.models import Q
from django.http import JsonResponse
from django.urls import path


class AjaxAdmin(admin.ModelAdmin):
    """
    This class is used to add ajax functionality to the admin interface.
    """

    def callback(self, request):
        """
        This method is used to handle ajax requests.
        """
        post = request.POST
        action = post.get("_action")
        selected = post.get("_selected")
        select_across = post.get("select_across")

        # call admin
        if hasattr(self, action):
            func, action, description = self.get_action(action)
            # 这里的queryset 会有数据过滤，只包含选中的数据
            queryset = self.get_changelist_instance(request).get_queryset(request)

            # 没有选择全部的时候才过滤数据
            if select_across == "0":
                if selected and selected.split(","):
                    queryset = queryset.filter(pk__in=selected.split(","))
            else:
                # 过滤搜索条件，自simpleui-2022.3.13版本开始，支持搜索条件过滤，simplepro需要3.4.2及以上版本
                # 只有选择全部的时候才过滤数据
                # 字段为_search和_filter 是为了防止命名冲突

                # search
                if "_search" in post:
                    search_fields = self.get_search_fields(request)

                    if search_fields:
                        search_value = post.get("_search")
                        if search_value:
                            q = Q()
                            for s in search_fields:
                                q = q | Q(**{s + "__icontains": search_value})
                            try:
                                queryset = queryset.filter(q)
                            except Exception as e:
                                traceback.print_exc()
                                raise e

                # filter条件过滤
                if "_filter" in post:
                    _filter = post.get("_filter")
                    if _filter:
                        filter_value = json.loads(_filter)
                        queryset = queryset.filter(**filter_value)

            return func(self, request, queryset)

    def field_button_callback(self, request, pk):
        """
        This method is used to handle ajax requests.
        """
        post = request.POST
        button = post.get("field_button")
        if button and hasattr(self, button):
            func = getattr(self, button)
            try:
                obj = self.model.objects.get(pk=pk)
            except Exception:
                return JsonResponse(msg='对象不存在', status='error')
            return func(request, obj)

    def get_layer(self, request):
        """
        This method is used to get the layer of the admin interface.
        """
        _action = request.GET.get("_action")
        if hasattr(self, _action):
            func, action, description = self.get_action(_action)
            if hasattr(func, "layer"):
                result = func.layer(self, request)
                return JsonResponse(data=result, safe=False)
        else:
            raise Exception(f'action "{_action}" not found')

    def get_urls(self):
        """
        This method is used to add ajax functionality to the admin interface.
        """

        def wrap(view):
            def wrapper(*args, **kwargs):
                return self.admin_site.admin_view(view)(*args, **kwargs)

            wrapper.model_admin = self
            return update_wrapper(wrapper, view)

        info = self.model._meta.app_label, self.model._meta.model_name

        return [
                   path("ajax", self.callback, name="%s_%s_ajax" % info),
                   path("layer", self.get_layer, name="%s_%s_layer" % info),
                   path("<path:pk>/ajax/", wrap(self.field_button_callback), name="%s_%s_change_ajax" % info),
               ] + super().get_urls()

==> django_simpleui_dev-2023.2.27-py3-none-any.whl: simpleui/admin.py <==
import json
import traceback
from functools import update_wrapper
from django.contrib import admin
from django.db.models import Q
from django.http import JsonResponse
from django.urls import path


class AjaxAdmin(admin.ModelAdmin):
    """
    This class is used to add ajax functionality to the admin interface.
    """
    pk_type = None

    def callback(self, request):
        """
        This method is used to handle ajax requests.
        """
        post = request.POST
        action = post.get("_action")
        selected = post.get("_selected")
        select_across = post.get("select_across")
        obj_id = post.get("obj_id")

        # call admin
        if hasattr(self, action):
            if obj_id:
                try:
                    if self.pk_type:
                        obj_id = self.pk_type(obj_id)
                    obj = self.model.objects.get(pk=obj_id)
                except Exception:
                    return JsonResponse({'msg': '对象不存在', 'status': 'error'})
                dispaly_list_func = getattr(self, action)
                try:
                    return dispaly_list_func(request, obj)
                except Exception as e:
                    return JsonResponse({'msg': str(e), 'status': 'error'})

            func, action, description = self.get_action(action)
            # 这里的queryset 会有数据过滤，只包含选中的数据
            queryset = self.get_changelist_instance(request).get_queryset(request)

            # 没有选择全部的时候才过滤数据
            if select_across == "0":
                if selected and selected.split(","):
                    queryset = queryset.filter(pk__in=selected.split(","))
            else:
                # 过滤搜索条件，自simpleui-2022.3.13版本开始，支持搜索条件过滤，simplepro需要3.4.2及以上版本
                # 只有选择全部的时候才过滤数据
                # 字段为_search和_filter 是为了防止命名冲突

                # search
                if "_search" in post:
                    search_fields = self.get_search_fields(request)

                    if search_fields:
                        search_value = post.get("_search")
                        if search_value:
                            q = Q()
                            for s in search_fields:
                                q = q | Q(**{s + "__icontains": search_value})
                            try:
                                queryset = queryset.filter(q)
                            except Exception as e:
                                traceback.print_exc()
                                raise e

                # filter条件过滤
                if "_filter" in post:
                    _filter = post.get("_filter")
                    if _filter:
                        filter_value = json.loads(_filter)
                        queryset = queryset.filter(**filter_value)

            try:
                return func(self, request, queryset)
            except Exception as e:
                return JsonResponse({'msg': str(e), 'status': 'error'})

    def field_button_callback(self, request, pk):
        """
        This method is used to handle ajax requests.
        """
        post = request.POST
        button = post.get("field_button")
        if button and hasattr(self, button):
            func = getattr(self, button)
            try:
                obj = self.model.objects.get(pk=pk)
            except Exception:
                return JsonResponse({'msg': '对象不存在', 'status': 'error'})
            try:
                return func(request, obj)
            except Exception as e:
                return JsonResponse({'msg': str(e), 'status': 'error'})

    def get_layer(self, request):
        """
        This method is used to get the layer of the admin interface.
        """
        _action = request.GET.get("_action")
        if hasattr(self, _action):
            func, action, description = self.get_action(_action)
            if hasattr(func, "layer"):
                result = func.layer(self, request)
                return JsonResponse(data=result, safe=False)
        else:
            raise Exception(f'action "{_action}" not found')

    def get_urls(self):
        """
        This method is used to add ajax functionality to the admin interface.
        """

        def wrap(view):
            def wrapper(*args, **kwargs):
                return self.admin_site.admin_view(view)(*args, **kwargs)

            wrapper.model_admin = self
            return update_wrapper(wrapper, view)

        info = self.model._meta.app_label, self.model._meta.model_name

        return [
                   path("ajax", self.callback, name="%s_%s_ajax" % info),
                   path("layer", self.get_layer, name="%s_%s_layer" % info),
                   path("<path:pk>/ajax/", wrap(self.field_button_callback), name="%s_%s_change_ajax" % info),
               ] + super().get_urls()
